/*
 *   file: form_address.c
 *  
 *  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libral.h>

#include "app.h"
#include "form.h"
#include "form_address.h"
#include "app.h"
#include "groups_cb.h"
#include "models.h"
#include "types.h"
#include "utils.h"


struct _RubricaFormAddressPrivate {
  RubricaGroupSet* set;

  GtkWidget* st;
  GtkWidget* num;
  GtkWidget* zip;
  GtkWidget* city;
  GtkWidget* prov;
  GtkWidget* state;
  GtkWidget* country;
  GtkWidget* add_type;
  GtkWidget* add_tree;
  GtkWidget* add_add;
  GtkWidget* add_del;
  GtkWidget* add_mod;
  
  gboolean dispose_has_run;
};


static void rubrica_form_address_class_init (RubricaFormAddressClass* klass);
static void rubrica_form_address_init       (RubricaFormAddress* obj);

static void rubrica_form_address_dispose    (RubricaFormAddress* obj);
static void rubrica_form_address_finalize   (RubricaFormAddress* obj);


static gboolean extract_addresses         (GtkTreeModel *model,
					   GtkTreePath *path,
					   GtkTreeIter *iter, gpointer data); 

static void     form_address_extract_data (RubricaFormAddress* form, 
					   RubricaCard *card);
static void     form_address_display_data (RubricaFormAddress* form, 
					   RubricaCard *card);
static void     form_address_clean_form   (RubricaFormAddress* form);


static RubricaFormClass* parent_class;


GType
rubrica_form_address_get_type()
{
  static GType form_address_type = 0;
  
  if (!form_address_type)
    {
      static const GTypeInfo form_address_info =
	{
	  sizeof(RubricaFormAddressClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_form_address_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaFormAddress),
	  0,
	  (GInstanceInitFunc) rubrica_form_address_init
	};

      form_address_type = g_type_register_static (RUBRICA_FORM_TYPE, 
						  "RubricaFormAddress",
						  &form_address_info, 0);
    }
  
  return form_address_type;
}


static void
rubrica_form_address_class_init(RubricaFormAddressClass* klass)
{
  GObjectClass *class;
 
  class  = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_form_address_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_form_address_finalize;

  parent_class = RUBRICA_FORM_CLASS(klass);
  parent_class->extract_data = (gpointer) form_address_extract_data;
  parent_class->display_data = (gpointer) form_address_display_data;
  parent_class->clean_form   = (gpointer) form_address_clean_form;
}


static void
rubrica_form_address_init(RubricaFormAddress* self)
{
  self->private = g_new(RubricaFormAddressPrivate, 1);
  
  self->private->st       = NULL;
  self->private->num      = NULL;
  self->private->zip      = NULL;
  self->private->city     = NULL;
  self->private->prov     = NULL;
  self->private->state    = NULL;
  self->private->country  = NULL;
  self->private->add_type = NULL;
  self->private->add_tree = NULL;
  self->private->add_add  = NULL;
  self->private->add_del  = NULL;
  self->private->add_mod  = NULL;

  self->private->dispose_has_run = FALSE;
}

 

static void 
rubrica_form_address_dispose (RubricaFormAddress* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  if (self->private->dispose_has_run)
    return;
  
  g_object_unref(self->private->st);
  g_object_unref(self->private->num);
  g_object_unref(self->private->zip);
  g_object_unref(self->private->city);
  g_object_unref(self->private->prov);
  g_object_unref(self->private->state);
  g_object_unref(self->private->country);
  g_object_unref(self->private->add_type);
  g_object_unref(self->private->add_tree);
  g_object_unref(self->private->add_add);
  g_object_unref(self->private->add_del);
  g_object_unref(self->private->add_mod);
  
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_form_address_finalize (RubricaFormAddress* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  g_free(self->private);
  self->private = NULL;
}


gboolean 
extract_addresses (GtkTreeModel *model, GtkTreePath *path,
		   GtkTreeIter *iter, gpointer data)
{
  RubricaCard* card = (RubricaCard*) data;
  RubricaAddress* address = NULL;
  RubricaAddressType adtype;
  gchar *type, *street, *number, *city;
  gchar *zip, *province, *state, *country;

  gtk_tree_model_get(model, iter, 
		     ADDRESS_STREET_COLUMN,   &street,
		     ADDRESS_NUMBER_COLUMN,   &number,
		     ADDRESS_CITY_COLUMN,     &city,
		     ADDRESS_TYPE_COLUMN,     &type,
		     ADDRESS_ZIP_COLUMN,      &zip,
		     ADDRESS_PROVINCE_COLUMN, &province,
		     ADDRESS_STATE_COLUMN,    &state,
		     ADDRESS_COUNTRY_COLUMN,  &country,
		     -1);
  adtype = rubrica_address_encode_type(type);

  address = rubrica_address_new();
  g_object_set(address, "address-type", adtype, "street", street, 
	       "street-number", number, "city", city, "zip", zip, 
	       "province", province, "state", state, 
	       "country", country, NULL);
  rubrica_card_add_address(RUBRICA_CARD(card), address);

  return FALSE;
}


static void 
form_address_extract_data (RubricaFormAddress* form, RubricaCard *card)
{
  GtkTreeModel* model = NULL;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->add_tree));
  if (model)
    {
      gtk_tree_model_foreach(GTK_TREE_MODEL(model), extract_addresses, card);
      g_object_unref(model);
    }
}


static void 
form_address_display_data (RubricaFormAddress* form, RubricaCard *card)
{
  GtkTreeView* tree;
  GtkTreeModel *model;
  GtkTreeIter iter;
  gpointer data;

  gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->add_type),
			   RUBRICA_ADDRESS_PREF);    

  tree  = (GtkTreeView*) form->private->add_tree;
  model = gtk_tree_view_get_model(tree);

  data = rubrica_card_get_address(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_address(RUBRICA_CARD(card)))
    {
      RubricaAddressType adtype;
      gchar *street, *number, *city, *type;
      gchar *zip, *province, *state, *country;
      
      g_object_get(RUBRICA_ADDRESS(data), "address-type", &adtype, 
		   "street", &street, "street-number", &number, 
		   "city", &city, "zip", &zip, "province", &province, 
		   "state", &state, "country", &country, NULL);
      type = rubrica_address_decode_type(adtype);

      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			 ADDRESS_STREET_COLUMN,   street,
			 ADDRESS_NUMBER_COLUMN,   number,
			 ADDRESS_CITY_COLUMN,     city,
			 ADDRESS_TYPE_COLUMN,     type,
			 ADDRESS_ZIP_COLUMN,      zip,
			 ADDRESS_PROVINCE_COLUMN, province,
			 ADDRESS_STATE_COLUMN,    state,
			 ADDRESS_COUNTRY_COLUMN,  country,
			 -1);  
    }
}


static void 
form_address_clean_form   (RubricaFormAddress* form)
{

}


/*  Callbacks
*/
void 
form_addresses_list_activated(GtkTreeSelection *selection, 
				      gpointer data)
{
  RubricaFormAddress* form = (RubricaFormAddress*) data;
  GtkTreeModel *model;
  GtkTreeIter iter;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->add_tree));

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      RubricaAddressType type;
      gchar *street, *number, *city, *zip;
      gchar *province, *state, *country;
      
      gtk_tree_model_get(model, &iter, 
			 ADDRESS_STREET_COLUMN,   &street,
			 ADDRESS_NUMBER_COLUMN,   &number,
			 ADDRESS_CITY_COLUMN,     &city,
			 ADDRESS_TYPE_COLUMN,     &type,
			 ADDRESS_ZIP_COLUMN,      &zip,
			 ADDRESS_PROVINCE_COLUMN, &province,
			 ADDRESS_STATE_COLUMN,    &state,
			 ADDRESS_COUNTRY_COLUMN,  &country,
			 -1);
      
      gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->add_type), type);
      gtk_entry_set_text(GTK_ENTRY(form->private->st),      street);
      gtk_entry_set_text(GTK_ENTRY(form->private->num),     number);
      gtk_entry_set_text(GTK_ENTRY(form->private->zip),     zip);
      gtk_entry_set_text(GTK_ENTRY(form->private->city),    city);
      gtk_entry_set_text(GTK_ENTRY(form->private->prov),    province);
      gtk_entry_set_text(GTK_ENTRY(form->private->state),   state);
      gtk_entry_set_text(GTK_ENTRY(form->private->country), country);
      
      gtk_widget_set_sensitive(form->private->add_mod, TRUE);
      gtk_widget_set_sensitive(form->private->add_del, TRUE);
    }
}


void
form_address_add_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  RubricaFormAddress *form = (RubricaFormAddress*) data;
  RubricaAddress* address;
  RubricaAddressType addtype;
  gchar *street, *number, *city, *zip;
  gchar *type, *prov, *state, *country;
  
  gtk_widget_set_sensitive(GTK_WIDGET(form->private->add_mod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->private->add_del), FALSE);

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->add_tree));
  g_return_if_fail(GTK_IS_TREE_MODEL(model));
  
  address = rubrica_address_new();
  
  addtype = gtk_combo_box_get_active(GTK_COMBO_BOX(form->private->add_type));
  street  = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->st)));
  number  = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->num)));
  city    = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->city)));
  zip     = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->zip)));
  prov    = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->prov)));
  state   = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->state)));
  country = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->country)));
  
  g_object_set(address, "address-type", addtype, "street", street, 
	       "street-number", number, "city", city, "zip", zip, 
	       "province", prov, "state", state, "country", country, NULL);

  type = rubrica_address_decode_type(addtype);

  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		     ADDRESS_STREET_COLUMN,   street,
		     ADDRESS_NUMBER_COLUMN,   number,
		     ADDRESS_TYPE_COLUMN,     type,
		     ADDRESS_CITY_COLUMN,     city,
		     ADDRESS_ZIP_COLUMN,      zip,
		     ADDRESS_PROVINCE_COLUMN, prov,
		     ADDRESS_STATE_COLUMN,    state,
		     ADDRESS_COUNTRY_COLUMN,  country,		     
		     -1);

  gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->add_type),
			   RUBRICA_ADDRESS_PREF);
  gtk_entry_set_text(GTK_ENTRY(form->private->st),     "");
  gtk_entry_set_text(GTK_ENTRY(form->private->num),    "");
  gtk_entry_set_text(GTK_ENTRY(form->private->zip),    "");
  gtk_entry_set_text(GTK_ENTRY(form->private->city),   "");
  gtk_entry_set_text(GTK_ENTRY(form->private->prov),   "");
  gtk_entry_set_text(GTK_ENTRY(form->private->state),  "");
  gtk_entry_set_text(GTK_ENTRY(form->private->country),"");
}


void
form_address_mod_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaAddress* address;
  RubricaFormAddress* form = (RubricaFormAddress*) data;
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->add_tree));
  g_return_if_fail(GTK_IS_TREE_MODEL(model));

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->private->add_tree));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gint i;
    GtkTreePath *path;
    gchar *street, *number, *city, *zip;
    gchar *type, *prov, *state, *country;
    RubricaAddressType addtype;

    /* create a new address and fill with form's data */
    address = rubrica_address_new();    

    addtype = gtk_combo_box_get_active(GTK_COMBO_BOX(form->private->add_type));
    street  = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->st)));
    number  = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->num)));
    city    = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->city)));
    zip     = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->zip)));
    prov    = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->prov)));
    state   = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->state)));
    country = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->country)));

    g_object_set(address, "address-type", addtype, "street", street, 
		 "street-number", number, "city", city, "zip", zip, 
		 "province", prov, "state", state, "country", country, NULL);

    type = rubrica_address_decode_type(addtype);
    
    /* remove old address from tree list 
     */    
    path = gtk_tree_model_get_path(model, &iter);
    i = gtk_tree_path_get_indices(path)[0];
    
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    gtk_tree_path_free(path);
    
    /* insert new address into the tree list 
     */
    gtk_list_store_insert(GTK_LIST_STORE(model), &iter, i);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		       ADDRESS_STREET_COLUMN,   street,
		       ADDRESS_NUMBER_COLUMN,   number,
		       ADDRESS_TYPE_COLUMN,     type,
		       ADDRESS_CITY_COLUMN,     city,
		       ADDRESS_ZIP_COLUMN,      zip,
		       ADDRESS_PROVINCE_COLUMN, prov,
		       ADDRESS_STATE_COLUMN,    state,
		       ADDRESS_COUNTRY_COLUMN,  country,	       
		       -1);

    /* clean form 
     */
    gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->add_type),
			     RUBRICA_ADDRESS_PREF);
    gtk_entry_set_text(GTK_ENTRY(form->private->st),      "");
    gtk_entry_set_text(GTK_ENTRY(form->private->num),     "");
    gtk_entry_set_text(GTK_ENTRY(form->private->zip),     "");
    gtk_entry_set_text(GTK_ENTRY(form->private->city),    "");
    gtk_entry_set_text(GTK_ENTRY(form->private->prov),    "");
    gtk_entry_set_text(GTK_ENTRY(form->private->state),   "");
    gtk_entry_set_text(GTK_ENTRY(form->private->country), "");		       
    
    gtk_widget_set_sensitive(GTK_WIDGET(form->private->add_mod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->private->add_del), FALSE);   
  }  
}


void
form_address_del_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormAddress* form = (RubricaFormAddress*) data;
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->add_tree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->private->add_tree));
  
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    
    gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->add_type),
			     RUBRICA_ADDRESS_PREF);
    gtk_entry_set_text(GTK_ENTRY(form->private->st),      "");
    gtk_entry_set_text(GTK_ENTRY(form->private->num),     "");
    gtk_entry_set_text(GTK_ENTRY(form->private->zip),     "");
    gtk_entry_set_text(GTK_ENTRY(form->private->city),    "");
    gtk_entry_set_text(GTK_ENTRY(form->private->prov),    "");
    gtk_entry_set_text(GTK_ENTRY(form->private->state),   "");
    gtk_entry_set_text(GTK_ENTRY(form->private->country), "");

    gtk_widget_set_sensitive(GTK_WIDGET(form->private->add_mod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->private->add_del), FALSE);   
  }  
}





/*   ***************************** Public *****************************
*/


/**
 * rubrica_form_address_new
 *
 * create a #RubricaFormAddress
 *
 * Returns: a new #RubricaFormAddress
 */
RubricaFormAddress*
rubrica_form_address_new(RubricaApp* app, GladeXML *gui)
{
  RubricaFormAddress* form;
  GtkTreeView *tree;
  GtkTreeModel *model;
  GtkTreeSelection *selection;

  form = g_object_new(rubrica_form_address_get_type(), NULL);
  
  form->private->st       = glade_xml_get_widget(gui, "st");
  form->private->num      = glade_xml_get_widget(gui, "num");
  form->private->zip      = glade_xml_get_widget(gui, "zip");
  form->private->city     = glade_xml_get_widget(gui, "city");
  form->private->prov     = glade_xml_get_widget(gui, "prov");
  form->private->state    = glade_xml_get_widget(gui, "state");
  form->private->country  = glade_xml_get_widget(gui, "country");

  form->private->add_type = glade_xml_get_widget(gui, "address_combo");
  form->private->add_tree = glade_xml_get_widget(gui, "addresses_tree");
  form->private->add_add  = glade_xml_get_widget(gui, "add_add");
  form->private->add_del  = glade_xml_get_widget(gui, "add_del");
  form->private->add_mod  = glade_xml_get_widget(gui, "add_mod");
  
  gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->add_type), 
			   RUBRICA_ADDRESS_PREF);  
  gtk_widget_set_sensitive(GTK_WIDGET(form->private->add_mod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->private->add_del), FALSE);
  
  /*   address model/view 
   */
  tree = (GtkTreeView *) form->private->add_tree;
  model = rubrica_model_new(ADDRESS_MODEL);
  gtk_tree_view_set_model(tree, GTK_TREE_MODEL(model));
  selection = gtk_tree_view_get_selection(tree);
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  rubrica_model_add_columns(tree, ADDRESS_COLUMNS);

  g_signal_connect(G_OBJECT(selection), "changed", 
		   G_CALLBACK(form_addresses_list_activated), form);  
  g_object_unref(model);
  
  /*    address buttons
  */  
  g_signal_connect (G_OBJECT (form->private->add_add), "clicked", 
		    G_CALLBACK (form_address_add_clicked), form);
  g_signal_connect (G_OBJECT (form->private->add_mod), "clicked", 
		    G_CALLBACK (form_address_mod_clicked), form); 
  g_signal_connect (G_OBJECT (form->private->add_del), "clicked", 
		    G_CALLBACK (form_address_del_clicked), form);
  
  return form;
}


/**
 * rubrica_form_address_free
 * @form: a #RubricaFormAddress
 * 
 * free the #RubricaFormAddress
 */
void
rubrica_form_address_free(RubricaFormAddress* form)
{
  g_return_if_fail(IS_RUBRICA_FORM(form));

  g_object_unref(form);   
}
